use std::rc::Rc;

use super::EditorViewState;
use anyhow::Error;
use web_sys::window;
use yew::AttrValue;

fn fetch_initial_editor_state_or_fail() -> anyhow::Result<EditorViewState> {
    let win = window().ok_or_else(|| Error::msg("Can't get window"))?;
    let local = win
        .local_storage()
        .map_err(|_| Error::msg("Error retrieving localStorage"))?
        .ok_or_else(|| Error::msg("Local storage not available"))?;

    let n_keys = local
        .length()
        .map_err(|_| Error::msg("localStorage broken"))?;
    let mut available_files: Vec<AttrValue> = vec![];
    for i in 0..n_keys {
        if let Some(key) = local
            .key(i)
            .map_err(|_| Error::msg("localStorage broken"))?
        {
            match key.strip_prefix("scriptfile_") {
                None => {}
                Some(filename) => {
                    available_files.push(filename.to_owned().into());
                }
            }
        }
    }

    let init_str: AttrValue = "init.lua".into();
    if !available_files.contains(&init_str) {
        available_files.push(init_str);
        local
            .set(
                "scriptfile_init.lua",
                "-- Put any code to run on every client load here.\n",
            )
            .map_err(|_| Error::msg("localStorage broken"))?;
    }

    Ok(EditorViewState {
        error_msg: None,
        available_files,
        open_file: "init.lua".into(),
        show_create_dialog: false,
        show_delete_dialog: None,
    })
}

pub(super) fn fetch_initial_editor_state() -> Rc<EditorViewState> {
    match fetch_initial_editor_state_or_fail() {
        Ok(s) => s.into(),
        Err(e) => EditorViewState {
            error_msg: Some(format!("Can't load editor data: {}", e).into()),
            available_files: vec![],
            open_file: "init.lua".into(),
            show_create_dialog: false,
            show_delete_dialog: None,
        }
        .into(),
    }
}

pub fn load_file_contents(file: &str) -> anyhow::Result<String> {
    let win = window().ok_or_else(|| Error::msg("Can't get window"))?;
    let local = win
        .local_storage()
        .map_err(|_| Error::msg("Error retrieving localStorage"))?
        .ok_or_else(|| Error::msg("Local storage not available"))?;
    Ok(local
        .get(&format!("scriptfile_{}", file))
        .map_err(|_| Error::msg("Error retrieving localStorage"))?
        .unwrap_or_else(|| "".to_owned()))
}

pub fn load_extant_file_contents(file: &str) -> anyhow::Result<String> {
    let win = window().ok_or_else(|| Error::msg("Can't get window"))?;
    let local = win
        .local_storage()
        .map_err(|_| Error::msg("Error retrieving localStorage"))?
        .ok_or_else(|| Error::msg("Local storage not available"))?;
    Ok(local
        .get(&format!("scriptfile_{}", file))
        .map_err(|_| Error::msg("Error retrieving localStorage"))?
        .ok_or_else(|| Error::msg("File not found"))?)
}
